/********************************************
 * UTIA AV CR, v.v.i
 * Signal Processing Department
 * Pod Vodarenskou vezi 4
 * 182 08 Prague
 * Czech Republic
 * web: http://zs.utia.cas.cz
 * contact: sp@utia.cz
 *
 * Filename: csc.c
 * Date: 09.12.2022
 * Author: Lukas Kohout <kohoutl@utia.cas.cz>
 * Description:
 *    V_PROC_SS CSC
  *
 *
 * Changelog:
 *
 *
 *
 *
 ********************************************/

#include "csc.h"


int csc_set_stream_param(csc_t *csc,
                         u16 dir,
                         u16 width,
                         u16 height,
                         XVidC_FrameRate fps,
                         XVidC_ColorFormat cfmt,
                         u16 interlaced
                        )
{
    XVidC_VideoMode video_mode;
    XVidC_VideoStream stream;
    XVidC_VideoTiming timing;
    XVidC_VideoTiming const *timing_p;

    video_mode = XVidC_GetVideoModeId(width, height, fps, interlaced);
    if (video_mode == XVIDC_VM_NOT_SUPPORTED) {
        video_mode = XVIDC_VM_CUSTOM;
        memset(&timing, 0, sizeof(XVidC_VideoTiming));
        timing.HActive = width;
        timing.VActive = height;
        timing_p = &timing;
    } else {
        timing_p = XVidC_GetTimingInfo(video_mode);
    }


    printf("Video mode %d: %dx%d ... ", video_mode, timing_p->HActive, timing_p->VActive);

//     printf("  HActive        : %d\n", timing_p->HActive       );
//     printf("  HFrontPorch    : %d\n", timing_p->HFrontPorch   );
//     printf("  HSyncWidth     : %d\n", timing_p->HSyncWidth    );
//     printf("  HBackPorch     : %d\n", timing_p->HBackPorch    );
//     printf("  HTotal         : %d\n", timing_p->HTotal        );
//     printf("  HSyncPolarity  : %d\n", timing_p->HSyncPolarity );
//     printf("  VActive        : %d\n", timing_p->VActive       );
//     printf("  F0PVFrontPorch : %d\n", timing_p->F0PVFrontPorch);
//     printf("  F0PVSyncWidth  : %d\n", timing_p->F0PVSyncWidth );
//     printf("  F0PVBackPorch  : %d\n", timing_p->F0PVBackPorch );
//     printf("  F0PVTotal      : %d\n", timing_p->F0PVTotal     );
//     printf("  F1VFrontPorch  : %d\n", timing_p->F1VFrontPorch );
//     printf("  F1VSyncWidth   : %d\n", timing_p->F1VSyncWidth  );
//     printf("  F1VBackPorch   : %d\n", timing_p->F1VBackPorch  );
//     printf("  F1VTotal       : %d\n", timing_p->F1VTotal      );
//     printf("  VSyncPolarity  : %d\n", timing_p->VSyncPolarity );


    /* Setup Video Processing Subsystem */
    stream.VmId           = video_mode;
    stream.Timing         = *timing_p;
    stream.ColorFormatId  = cfmt;
    stream.ColorDepth     = (XVidC_ColorDepth)csc->inst.Csc.Config.MaxDataWidth;
    stream.PixPerClk      = (XVidC_PixelsPerClock)csc->inst.Csc.Config.PixPerClk;
    stream.FrameRate      = fps;
    stream.IsInterlaced   = interlaced;

    if (dir == CSC_STREAM_IN)
        csc->stream_in = stream;
    else
        csc->stream_out = stream;


    return XST_SUCCESS;
}


int csc_setup_mode(csc_t *csc)
{
    int status = XST_SUCCESS;

    // when setting up a new resolution, start with default picture settings
    XV_CscSetPowerOnDefaultState(&(csc->inst));

    // set the proper color depth: get it from the vprocss config
    XV_CscSetColorDepth(&(csc->inst), (XVidC_ColorDepth)csc->inst.Csc.Config.MaxDataWidth);

    // all other picture settings are filled in by XV_CscSetColorspace
    status = XV_CscSetColorspace(&(csc->inst),
                        csc->stream_in.ColorFormatId, //CscIn,
                        csc->stream_out.ColorFormatId,  //CscOut,
                        csc->inst.StandardIn, //StdIn,
                        csc->inst.StandardOut,  //StdOut,
                        csc->inst.OutputRange //RangeOut
                        );
    if (status != XST_SUCCESS) return status;

    // set the Global Window size
    XV_CscSetActiveSize(&(csc->inst),
                        csc->stream_out.Timing.HActive, //WidthOut,
                        csc->stream_out.Timing.VActive  //HeightOut
                        );

    /* Start Csc sub-core */
    XV_CscStart(&csc->inst);
    return(status);
}


/*
 * initialize
 */
int csc_init_common(csc_t *csc, XV_csc_Config *csc_config)
{

    int status;
    XV_csc_Config *config = csc_config;

    config->BaseAddress = csc->base_addr;

//     csc->inst.Csc.Config.BaseAddress = csc->base_addr;
//     csc->inst.Csc.IsReady = XIL_COMPONENT_IS_READY;


    // initialize CSC
    status = XV_csc_CfgInitialize(&(csc->inst.Csc), config, config->BaseAddress);
    if (status != XST_SUCCESS) {
        printf( "Initialization failed %d ... ", status);
        return -1;
    }
//    printf(" 0x%08X ", csc->inst.Csc.Config.BaseAddress);


   XV_CscStop(&(csc->inst));
//      *(volatile uintptr_t*)((csc->inst.Csc.Config.BaseAddress) + (XV_CSC_CTRL_ADDR_AP_CTRL)) = 0;
//      *(volatile uintptr_t*)((csc->base_addr) + (XV_CSC_CTRL_ADDR_AP_CTRL)) = 0;

    return 0;
}

