#include <stdio.h>
#include <stdlib.h>

#include "hdmi_io_ctrl.hpp"

#define NUMROWS 	1080
#define NUMCOLS 	1920
#define NUMPADCOLS 	2048

const char *i2c_im_device = "/dev/i2c-1";


#include <signal.h>


unsigned system_stop = 0;
struct sigaction int_handler;

// signal handler for CTRL+c
void sig_handler(int signo)
{
    if (signo == SIGINT) {
        printf("\nCTRL-C pressed, terminating ...\n");
        system_stop = 1;
    }
}


#include <opencv2/core.hpp>
#include <opencv2/highgui.hpp>
#include <opencv2/imgproc.hpp>
#include <vitis/ai/facedetect.hpp>


//****************************************************************************


void process_result(cv::Mat &m1, const vitis::ai::FaceDetectResult &result) {
  for (const auto &r : result.rects) {
    cv::rectangle(m1,
                  cv::Rect{cv::Point(r.x * m1.cols, r.y * m1.rows),
                           cv::Size{(int)(r.width * m1.cols),
                                    (int)(r.height * m1.rows)}},
                  CV_RGB(0, 0xff,0) ,2);
  }
}


//****************************************************************************
int main(void)
{



    printf("\n\n*****************************************************************************\n");
    printf(    "*  Signal Processing Dept., UTIA AV CR, v.v.i.                              *\n");
    printf(    "*  HDMI In/Out Application on AVNET FMC Board - Petalinux                   *\n");
    printf(    "*****************************************************************************\n\n");



    /*
     * register signal handler (CTRL+C)
     */
    int_handler.sa_handler = sig_handler;
    if ( signal(SIGINT, sig_handler) == SIG_ERR )  printf("Cannot catch SIGINT\n");
    sigaction(SIGINT, &int_handler, 0);

    /*
     * Setup facedetect network
     */
    auto network = vitis::ai::FaceDetect::create("densebox_640_360", true);

    printf("Network input: %d x %d\r\n", network->getInputHeight(), network->getInputWidth());


    hdmi_io_ctrl video(1, i2c_im_device, 7, 75.0f, &system_stop);


    video.hdmi_out_start();

#if (1)
    while (video.hdmi_in_start()) {
    	printf("Input HDMI signal not detected ... \n");
    	sleep(1);
    	if (system_stop) return 1;
    }
#endif
	int frame;
	while (system_stop == 0) {
#if (0)
		frame = video.fb_stack->pop();
		video.hdmi_out_show(frame);
#else
		if (video.hdmi_in_capture(frame)) {
			printf("ERROR: Failed to capture frame\r\n");
			video.vdma_print_rx_err(video.vdma_get_rx_err());
			/*
			 * try to restart hdmi input
			 */
		    while (video.hdmi_in_start()) {
		    	printf("Input HDMI signal not detected ... \n");
		    	sleep(1);
		    }
		} else {

			/*
			 * Get frame address
			 */
			uintptr_t *fptr_in = video.get_frame_addr(frame);

			/*
			 * Invalidate cache to process frame by CPU
			 */
			video.invalidate_frame(frame);

			cv::Mat image(NUMROWS, NUMCOLS, CV_8UC3, fptr_in, NUMPADCOLS*4);

			/*
			 * Use facedetect
			 */
			 auto result = network->run(image);
			// auto result2 = networt->run_with_xrt_bo(			       const std::vector<vart::xrt_bo_t>& input_bos) = 0;

			 process_result(image, result);

			 /*
			  * Show input frame with drawn ovelay
			  */
			video.flush_frame(frame);
			video.hdmi_out_show(frame);
		}
#endif
	}

    return 0;
}


