----------------------------------------------------------------------------------------
-- UTIA AV CR, v.v.i
-- Signal Processing Department
-- Pod Vodarenskou vezi 4
-- 182 08 Prague
-- Czech Republic
-- web: http://zs.utia.cas.cz
-- contact: sp@utia.cz
--
-- Filename: hblank_det.vhd
-- Date: 09.26.2022
-- Author: Lukas Kohout <kohoutl@utia.cas.cz>
-- Description:
--
--
-- Changelog:
--
--
--
--
----------------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

------------------------------------------------------------------------------
------------------------------------------------------------------------------
entity hblank_det is
    generic (
        C_CLK_FREQ_HZ     : natural := 10000000
    );
    port (
        clk               : in  std_logic;
        rstn              : in  std_logic;

        hblank            : in  std_logic;

        locked            : out std_logic
    );

end entity hblank_det;

------------------------------------------------------------------------------
architecture rtl of hblank_det is


------------------------------------------------------------------------------
    constant LOCK_TH : natural := C_CLK_FREQ_HZ / 8; -- 125 ms
--     constant LOCK_TH : natural := 8;

    signal s_lock_cnt : natural range 0 to LOCK_TH;

    signal s_hblank_ff  : std_logic_vector(1 downto 0);
    signal s_hblank_re  : std_logic;
    signal s_hblank_fe  : std_logic;


    signal s_locked   : std_logic;


------------------------------------------------------------------------------
begin

------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- FFs

    p_ffs : process(clk) is
    begin
        if clk'event and clk = '1' then
            if rstn = '0' then
                s_hblank_ff <= (others => '0');
            else
                s_hblank_ff <= s_hblank_ff(0) & hblank;
            end if;
        end if;
    end process p_ffs;


    s_hblank_re <= s_hblank_ff(0) and not s_hblank_ff(1);
    s_hblank_fe <= s_hblank_ff(1) and not s_hblank_ff(0);

------------------------------------------------------------------------------
--
    p_lock_cnt : process (clk) is
    begin
        if clk'event and clk = '1' then
            if (rstn = '0') or (s_hblank_re = '1') or (s_hblank_fe = '1') then
                s_lock_cnt <= 0;
                s_locked <= '1';
            else
                if s_lock_cnt < LOCK_TH then
                    s_lock_cnt <= s_lock_cnt + 1;
                    s_locked <= '1';
                else
                    s_locked <= '0';
                end if;
            end if;
        end if;
    end process p_lock_cnt;


    locked <= s_locked;

end architecture rtl;

